#ifdef HAVE_CONFIG_H
#include <config.h>
#endif



#if (defined(__FreeBSD__) || defined(__APPLE__))
#include <sys/cdefs.h>
#include <sys/syslimits.h>
#else
#include <linux/limits.h>   /* PATH_MAX */
#endif


#include <sys/stat.h>
#include <fcntl.h>      /* open() */
#include <errno.h>
#include <unistd.h>     /* getcwd(), chdir(), close() */
#include <stdarg.h>     /* va_start(), va_end(), va_list */
#include <string.h>     /* strncpy(), strncat() */
#include <glob.h>       /* glob(), globfree() */

#include <linux/adwin.h>
#include "libadwin/adctl.h"



#include <stdlib.h>
#include <pwd.h>
#include <stdio.h>


static int __adiogdevnum(int *n)
{
    glob_t globbuf;


    if (glob("/proc/adwin/dev/0x*", 0, NULL, &globbuf))
        *n = 0;
    else
        *n = globbuf.gl_pathc;

    globfree(&globbuf);

    return 0;
}

static int __adiogdevconf(struct adconf *adc)
{
    int i;
    char cwd[PATH_MAX];
    glob_t globbuf;
    struct adreq *adr;

    if (! adc) {
        errno = ENOMEM;
        return -1;
    }

    if (! adc->adc_len) {
        int n = 0;
        __adiogdevnum(&n);
        adc->adc_len = n * sizeof(struct adreq);
        return 0;
    }

    adr = (struct adreq *)adc->adc_buf;
    getcwd(cwd, PATH_MAX);

    if (chdir("/proc/adwin/dev"))
        perror("chdir /proc/adwin/dev/");

    glob("0x*", 0, NULL, &globbuf);
    for (i = 0; (char*)(adr + i) < (char *)(adc->adc_buf + adc->adc_len); i++) {
        strncpy(adr[i].adr_name, globbuf.gl_pathv[i], ADWIN_DEVNAMESIZE);
    }
    globfree(&globbuf);

    chdir(cwd);
    return 0;
}


static int __adioctl(int request, struct adreq *adr)
{
    int fd;
    char dev[32] = { "/proc/adwin/dev/" };

    fd = open(strncat(dev, adr->adr_name, ADWIN_DEVNAMESIZE), O_RDWR);
    if (fd < 0)
        goto __error_return;

    if (ioctl(fd, request, &adr->adr_buf) < 0)
        goto __error_return;

    close(fd);
    return 0;

__error_return:
    /* adwin_errno = -errno; */
    close(fd);
    return -1;
}

/* NUMBER MAPPING ... */
struct {
    int adctl, ioctl;
} ctlmap[] = {
    { ADIOGDEVDESC,     ADIOGKDEVDESC },
    { ADIOSDEVDESC,     ADIOSKDEVDESC },
    { ADIOGDEVUID,      ADIOGKDEVUID },
    { ADIOSDEVUID,      ADIOSKDEVUID },
    { ADIOGDEVGID,      ADIOGKDEVGID },
    { ADIOSDEVGID,      ADIOSKDEVGID },
    { ADIOGDEVMODE,     ADIOGKDEVMODE },
    { ADIOSDEVMODE,     ADIOSKDEVMODE },
    { ADIOGDEVTYPE,     ADIOGKDEVTYPE },
    { ADIOSDEVTYPE,     ADIOSKDEVTYPE },
    { ADIOGDEVBASE,     ADIOGKDEVBASE },
    { ADIOSDEVBASE,     ADIOSKDEVBASE },
    { ADIOGDEVSERIAL,   ADIOGKDEVSERIAL },
    { ADIOSDEVSERIAL,   ADIOSKDEVSERIAL },
    { ADIOGDEVSERVER,   ADIOGKDEVSERVER },
    { ADIOSDEVSERVER,   ADIOSKDEVSERVER },
    { ADIOGDEVTIMEO,    ADIOGKDEVTIMEO },
    { ADIOSDEVTIMEO,    ADIOSKDEVTIMEO },
    { ADIOGDEVCOUNT,    ADIOGKDEVCOUNT },
    { ADIOSDEVCOUNT,    ADIOSKDEVCOUNT },
    { ADIOGDEVHSTLNK,   ADIOGKDEVHSTLNK },
    { ADIOSDEVHSTLNK,   ADIOSKDEVHSTLNK },
    { ADIOGDEVPW,       ADIOGKDEVPW },
    { ADIOSDEVPW,       ADIOSKDEVPW },
    { 0, 0 }
};

int adctl(int request, ...)
{
    int res;
    va_list arg;

    va_start(arg, request);

    if (request == ADIOGDEVNUM) {
        res = __adiogdevnum(va_arg(arg, int *));
        va_end(arg);
        return res;
    } else if (request == ADIOGDEVCONF) {
        res = __adiogdevconf(va_arg(arg, struct adconf*));
        va_end(arg);
        return res;
    } else {
        int i;
        for (i = 0; ctlmap[i].adctl; i++)
            if (request == ctlmap[i].adctl) {
                res = __adioctl(ctlmap[i].ioctl, va_arg(arg, struct adreq*));
                va_end(arg);
                return res;
            }
    }

    va_arg(arg, struct adreq *);
    va_end(arg);
    errno = ENOTTY;
    return -1;

}
/*
int main(void) {
    int n;
    struct adconf adc;

    adctl(ADIOGDEVNUM, &n);
    printf("ADIOGDEVNUM: %d\n", n);

    adc.adc_len = 0;
    adctl(ADIOGDEVCONF, &adc);

    if (! adc.adc_len) {
        fprintf(stderr, "Keine Geraete definiert!\n");
        exit(0);
    }

    adc.adc_buf = malloc(adc.adc_len);
    adctl(ADIOGDEVCONF, &adc);

    do {
        n--;
        printf("%s\n", adc.adc_req[n].adr_name);

        adctl(ADIOGDEVDESC, &adc.adc_req[n]);
        printf("\tADIOGDEVDESC: %s\n", adc.adc_req[n].adr_desc);

        adctl(ADIOGDEVUID, &adc.adc_req[n]);
        printf("\tADIOGDEVUID:  %d\n", adc.adc_req[n].adr_uid);
        adctl(ADIOGDEVGID, &adc.adc_req[n]);
        printf("\tADIOGDEVGID:  %d\n", adc.adc_req[n].adr_gid);
        adctl(ADIOGDEVMODE, &adc.adc_req[n]);
        printf("\tADIOGDEVMODE: %o\n", adc.adc_req[n].adr_mode);
    } while(n > 0);

    free(adc.adc_buf);

    exit(0);
}
*/
