#! /usr/bin/python
# Python2.x example for ADwin-Python-Module

from ADwin import ADwin, ADwinError
import sys, os, time, ctypes, array

DEVICENUMBER = 1
RAISE_EXCEPTIONS = 1
PROCESSORTYPE = '9' # '9' for T9, '10' for T10, '11' for T11
PROCESS = 'example.t91' # T9
#PROCESS = 'example.ta1'    # T10
#PROCESS = 'example.tb1'   # T11

# if you need help: help(ADwin)
try:
    adw = ADwin(DEVICENUMBER, RAISE_EXCEPTIONS)
    
    # boot then ADwin-system, load process and start it
    if os.name == 'posix':
        adw.Boot('adwin'+PROCESSORTYPE+'.btl')
    else:
        adw.Boot(adw.ADwindir + 'adwin'+PROCESSORTYPE+'.btl')
    
    proc = os.path.abspath(os.path.dirname(sys.argv[0])) + os.sep + PROCESS
    adw.Load_Process(proc)
    
    adw.Start_Process(1)
    time.sleep(0.3) # give init time to setup the datas
    
    print 'Process_Status:', adw.Process_Status(1)
    print 'Processor_Type:', adw.Processor_Type()
    print 'Get_Processdelay:', adw.Get_Processdelay(1)
    print 'Test_Version:', adw.Test_Version()
    
    # Long-Parameter
    for i in range(1, 81): adw.Set_Par(i, i)
    print 'Get_Par_All:', adw.Get_Par_All()[:]
    print 'Get_Par_Block:', adw.Get_Par_Block(10, 10)[:]
    
    print 'Get_Par:',
    for i in range(10): print adw.Get_Par(1),
    
    # Float-Parameter
    for i in range(1, 81): adw.Set_FPar(i, i+0.12345)
    print '\nGet_FPar_All:', adw.Get_FPar_All()[:]
    print 'Get_FPar_Block:', adw.Get_FPar_Block(10, 10)[:]
    
    print 'Get_FPar:',
    for i in range(1,10): print adw.Get_FPar(i),
    
    # long-Data
    print '\nData_Length:', adw.Data_Length(1)
    # type ctypes.c_int32
    dataType = ctypes.c_int32 * 100
    a = dataType(0)
    for i in range(100): a[i] = i+100
    adw.SetData_Long(a, 1, 1, 100)
    
    # also possible:
    # type list: adw.SetData_Long(range(100, 200), 1, 1, 100) 
    # type array: adw.SetData_Long(array.array('l', range(100,200)), 1, 1, 100)
    
    b = adw.GetData_Long(1, 10, 20)
    print 'GetData_Long:', b[:]
    
    # Float-Data
    print 'Data_Length:', adw.Data_Length(2)
    # type ctypes.c_float
    dataType = ctypes.c_float * 100
    a = dataType(0)
    for i in range(100): a[i] = i+100.1234
    adw.SetData_Float(a, 2, 1, 100)
    c = adw.GetData_Float(2, 10, 20)
    print 'GetData_Float:', c[:]
    
    #Fifo
    print 'Fifo_Full:', adw.Fifo_Full(3)
    adw.Fifo_Clear(3)
    print 'Fifo_Empty:', adw.Fifo_Empty(3)
    adw.SetFifo_Long(3, b, 20)
    print 'Fifo_Full:', adw.Fifo_Full(3)
    print 'GetFifo_Long', adw.GetFifo_Long(3, 10)[:]
    print 'Fifo_Full:', adw.Fifo_Full(3)
    
    # String
    s = "The answer to this is very simple. It was a joke. \nIt had to be a number, an ordinary, smallish number, and I chose that one.\n" \
        +"Binary representations, base thirteen, Tibetan monks are all complete nonsense.\n" \
        +"I sat at my desk, stared into the garden and thought '42 will do' I typed it out. End of story."
    adw.SetData_String(5, s)
    
    s, Count = adw.GetData_String(5, 400)
    print 'GetData_String:', s.value
    print 'Count of GetData_String:', Count

    adw.Stop_Process(1)
    print 'Free_Mem:', adw.Free_Mem(1), 'Bytes, local program memory'
    adw.Clear_Process(1)
    print 'Free_Mem:', adw.Free_Mem(1), 'Bytes, local program memory after Clear_Process'
    
except ADwinError, e:
    print '***', e
